package com.css.simulation.resource.scheduler.api.consumer;


import api.common.util.JsonUtil;
import com.css.simulation.resource.scheduler.api.model.ProjectMessageModel;
import com.css.simulation.resource.scheduler.application.service.ProjectService;
import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.ObjectMapper;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.apache.kafka.clients.consumer.ConsumerRecord;
import org.springframework.kafka.annotation.KafkaListener;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;

@Component
@Slf4j
public class ProjectConsumer {
    @Resource
    private ProjectService projectService;

    /**
     * @param projectRecord 项目启动消息
     */
    @KafkaListener(groupId = "simulation-resource-scheduler", topics = "${custom.mq-start-project-topic}")
    @SneakyThrows
    public void acceptMessage(ConsumerRecord<String, String> projectRecord) {
        log.info("接收到项目开始消息为：" + projectRecord);
        ProjectMessageModel projectMessageModel = JsonUtil.jsonToBean(projectRecord.value(), ProjectMessageModel.class);
        projectService.runProject(projectMessageModel);
    }

    //* -------------------------------- 结束 --------------------------------

    @KafkaListener(groupId = "simulation-resource-scheduler", topics = "${custom.mq-stop-project-topic}")
    @SneakyThrows
    public void stopProject(ConsumerRecord<String, String> stopRecord) {
        log.info("接收到的项目终止消息为：" + stopRecord);
        JsonNode jsonNode = new ObjectMapper().readTree(stopRecord.value());
        String projectId = jsonNode.path("projectId").asText();
        String projectType = jsonNode.path("type").asText();

        projectService.stopProject(projectType, projectId);
    }


}
