package com.css.simulation.resource.scheduler.infrastructure.scheduler;

import api.common.pojo.constants.DictConstants;
import api.common.util.CollectionUtil;
import com.css.simulation.resource.scheduler.api.model.ProjectMessageModel;
import com.css.simulation.resource.scheduler.application.entity.ProjectWaitQueueEntity;
import com.css.simulation.resource.scheduler.application.service.ProjectService;
import com.css.simulation.resource.scheduler.domain.service.ProjectDomainService;
import com.css.simulation.resource.scheduler.infrastructure.entity.ProjectEntity;
import lombok.extern.slf4j.Slf4j;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.util.Collection;
import java.util.List;

@Component
@Slf4j
public class ProjectScheduler {
    @Resource
    private ProjectService projectService;
    @Resource
    private ProjectDomainService projectDomainService;

    /**
     * 调度项目启动
     */
    @Scheduled(fixedDelay = 60000)
    public void dispatchProject() {
        List<ProjectWaitQueueEntity> projectWaitQueue = projectDomainService.getWaitQueue();
        if (CollectionUtil.isNotEmpty(projectWaitQueue)) {
            ProjectWaitQueueEntity projectWaitQueueEntity = projectWaitQueue.get(0);
            String waitingType = projectWaitQueueEntity.getWaitingType();
            if (DictConstants.PROJECT_WAIT_TYPE_EXECUTE.equals(waitingType)) {
                log.info("尝试启动等待中的项目：{}", projectWaitQueueEntity);
                final ProjectMessageModel projectMessageModel = projectWaitQueueEntity.getProjectMessageModel();
                projectService.checkIfCanRun(projectMessageModel);
            } else if (DictConstants.PROJECT_WAIT_TYPE_EXPAND.equals(waitingType)) {
                log.info("尝试扩充运行中的项目：{}", projectWaitQueueEntity);
                ProjectMessageModel projectMessageModel = projectWaitQueueEntity.getProjectMessageModel();
                String projectId = projectMessageModel.getProjectId();
                ProjectEntity project = projectDomainService.getProjectByProjectId(projectId);
                if (DictConstants.PROJECT_COMPLETED.equals(project.getNowRunState())) {
                    log.info("等待扩充的项目" + projectWaitQueueEntity + "已运行完成，无需扩充。");
                    projectWaitQueue.remove(0);
                    projectDomainService.setWaitQueue(projectWaitQueue);
                }
            }
        }
    }

}
